@@ -16,12 +16,7 @@
 # limitations under the License.
 #
 
-$:.unshift(File.dirname(__FILE__)) unless
-  $:.include?(File.dirname(__FILE__)) || $:.include?(File.expand_path(File.dirname(__FILE__)))
-
+require 'ohai/version'
 require 'ohai/config'
 require 'ohai/system'
 
-module Ohai
-  VERSION = '0.3.2'
-end
