@@ -17,7 +17,7 @@
 #
 
 
-require File.join(File.dirname(__FILE__),  '..', '..', '..', '/spec_helper.rb')
+require File.expand_path(File.dirname(__FILE__) + '/../../../spec_helper.rb')
 
 describe Ohai::System, "FreeBSD plugin platform" do
   before(:each) do
