@@ -16,7 +16,7 @@
 # limitations under the License.
 #
 
-require File.dirname(__FILE__) + '/spec_helper.rb'
+require File.expand_path(File.dirname(__FILE__) + '/spec_helper.rb')
 
 describe Ohai do
   
